from qgis.PyQt.QtCore import QT_TRANSLATE_NOOP

from qgis.core import (
  QgsProcessingParameterBoolean,
  QgsProcessingParameterString,
  QgsProcessingUtils
  )

import os
import zipfile
import tarfile
import requests

from ..algabstract import algabstract

class installcomponents(algabstract):
  

  PARAMETERS = {                  
    "INSTALL_NM": {
      "ui_func": QgsProcessingParameterBoolean,
      "ui_args": {
        "description": QT_TRANSLATE_NOOP("installcomponents","Download and Install NoiseModelling" ),
        "defaultValue": True
      }
    },
    "INSTALL_JAVA": {
      "ui_func": QgsProcessingParameterBoolean,
      "ui_args": {
        "description": QT_TRANSLATE_NOOP("installcomponents","Download and Install Java environment required for NoiseModelling" ),
        "defaultValue": True
      }
    },
    "URL_NM": {
      "advanced": True,
      "ui_func": QgsProcessingParameterString,
      "ui_args": {
        "description": QT_TRANSLATE_NOOP("installcomponents","Path of NoiseModelling" ),
        "defaultValue": "https://github.com/Universite-Gustave-Eiffel/NoiseModelling/releases/download/v4.0.5/NoiseModelling_without_gui.zip"
      }
    },
    "URL_JAVA": {
      "advanced": True,
      "ui_func": QgsProcessingParameterString,
      "ui_args": {
        "description": QT_TRANSLATE_NOOP("installcomponents","Path of Java environment" ),
        "defaultValue": ""
      }
    },
    "PATH_NM": {
      "advanced": True,
      "ui_func": QgsProcessingParameterString,
      "ui_args": {
        "description": QT_TRANSLATE_NOOP("installcomponents","Path of NoiseModelling" ),
        "defaultValue": ""
      }
    },
    "PATH_JAVA": {
      "advanced": True,
      "ui_func": QgsProcessingParameterString,
      "ui_args": {
        "description": QT_TRANSLATE_NOOP("installcomponents","Path of Java environment" ),
        "defaultValue": ""
      }
    }
  }
  
  
  def __init__(self, nm_version = "4.0.5", java_version = "11.0.2"):
    super().__init__()
        
    if self.ENV["PLATFORM"] == 'Windows':
      self.PARAMETERS["URL_JAVA"]["ui_args"]["defaultValue"]  = "https://download.java.net/java/GA/jdk11/9/GPL/openjdk-11.0.2_windows-x64_bin.zip"
      self.PARAMETERS["PATH_NM"]["ui_args"]["defaultValue"]  = os.path.join(os.path.normpath(os.environ['APPDATA']), "H-RISK", "NoiseModelling", nm_version)
      self.PARAMETERS["PATH_JAVA"]["ui_args"]["defaultValue"] = os.path.join(os.path.normpath(os.environ['APPDATA']), "H-RISK", "Java", java_version)
    elif self.ENV["PLATFORM"] == 'Darwin':
      self.PARAMETERS["URL_JAVA"]["ui_args"]["defaultValue"]  = "https://download.java.net/java/GA/jdk11/9/GPL/openjdk-11.0.2_osx-x64_bin.tar.gz"
      self.PARAMETERS["PATH_NM"]["ui_args"]["defaultValue"]  = os.path.join("~", "Applications", "H-RISK", "NoiseModelling", nm_version)
      self.PARAMETERS["PATH_JAVA"]["ui_args"]["defaultValue"] = os.path.join("~", "Applications", "H-RISK", "Java", java_version)
    elif self.ENV["PLATFORM"] == 'Linux':
      self.PARAMETERS["URL_JAVA"]["ui_args"]["defaultValue"]  = "https://download.java.net/java/GA/jdk11/9/GPL/openjdk-11.0.2_linux-x64_bin.tar.gz"
      self.PARAMETERS["PATH_NM"]["ui_args"]["defaultValue"]  = os.path.join("~", "H-RISK", "NoiseModelling", nm_version)
      self.PARAMETERS["PATH_JAVA"]["ui_args"]["defaultValue"] = os.path.join("~", "H-RISK", "Java", java_version)
  
  def initAlgorithm(self, config):
    self.initParameters()
    
  def processAlgorithm(self, parameters, context, feedback):
    # set a temporary folder path
    tmp_dir = os.path.normpath(os.path.dirname(QgsProcessingUtils.generateTempFilename("")))
    if not os.path.exists(tmp_dir):
      os.makedirs(tmp_dir)

    # set a variable to store a path of wps_scripts and java.exe
    wpsscript_path = None
    javaexe_path = None
    
    # if installing NoiseModelling
    if self.parameterAsBoolean(parameters, "INSTALL_NM", context):
      try:
        
        nm_download_path = self.downloadFile(
          session = requests.Session(), 
          request_args = {
            "method": "GET",
            "url": self.parameterAsString(parameters, "URL_NM", context) 
            }, 
          download_msg = "Downloading NoiseModelling...", 
          feedback_progress = True, feedback = feedback)
        
        # extract NoiseModelling
        feedback.pushInfo(self.tr("Extracting NoiseModelling..."))
        nm_folder = self.parameterAsString(parameters, "PATH_NM", context)
        if not os.path.exists(nm_folder):
          os.makedirs(nm_folder)
        self.extractArchive(nm_download_path, feedback, extension=".zip", extract_dir=nm_folder)
        
        # find a path of wps_scripts
        for root, dirs, files in os.walk(nm_folder):
          if "wps_scripts" in files:
            wpsscript_path = root
            break
        
        # set a environment variable of NOISEMODELLING_HOME
        feedback.pushInfo(self.tr("Setting an environment variable of NoiseModelling..."))
        self.setEnvVariable("NOISEMODELLING_HOME", os.path.dirname(wpsscript_path))
          
      except Exception as e:
        # if any error occurred
        feedback.reportError(self.tr("Error(s) occurred at installing NoiseModelling."), fatalError=True)
        raise e
    
    # if installing Java
    if self.parameterAsBoolean(parameters, "INSTALL_Java", context):
      try:
        
        java_download_path = self.downloadFile(
          session = requests.Session(), 
          request_args = {
            "method": "GET",
            "url": self.parameterAsString(parameters, "URL_JAVA", context) 
            }, 
          download_msg = "Downloading Java environment...", 
          feedback_progress = True, feedback = feedback) 
                
        java_extension = ".zip" if self.ENV["PLATFORM"] == 'Windows' else ".tar.gz"
          
        
        # extract Java
        feedback.pushInfo(self.tr("Extracting Java environment..."))
        java_folder = self.parameterAsString(parameters, "PATH_JAVA", context)
        if not os.path.exists(java_folder):
          os.makedirs(java_folder)
        self.extractArchive(java_download_path, feedback, extension=java_extension, extract_dir=java_folder)
        
        # find a path of java.exe
        java_exec = "java.exe" if self.ENV["PLATFORM"] == 'Windows' else "java"
        for root, dirs, files in os.walk(java_folder):
          if java_exec in files:
            javaexe_path = root
            break
        
        # set a environment variable of JAVA_FOR_NOISEMODELLING
        feedback.pushInfo(self.tr("Setting an environment variable of Java..."))
        self.setEnvVariable("JAVA_FOR_NOISEMODELLING", os.path.dirname(javaexe_path))
        
      except Exception as e:
        # if any error occurred
        feedback.reportError(self.tr("Error(s) occurred at installing Java environment."), fatalError=True)
        raise e
    
    if wpsscript_path is not None or javaexe_path is not None:        
      feedback.pushInfo(self.tr("Environment variables are set as follows. Restart QGIS to apply changes."))
    
    output_dict = {}
    if wpsscript_path is not None:
      feedback.pushInfo(f"NOISEMODELLING_HOME: {os.path.dirname(wpsscript_path)}")
      output_dict.update({"NOISEMODELLING_HOME": os.path.dirname(wpsscript_path)})
    
    if javaexe_path is not None:
      feedback.pushInfo(f"JAVA_FOR_NOISEMODELLING: {os.path.dirname(javaexe_path)}")
      output_dict.update({"JAVA_FOR_NOISEMODELLING": os.path.dirname(javaexe_path)})
    return output_dict
  
  
  def createInstance(self):
    return installcomponents()


  def displayName(self):
    return self.tr("Install required components")

  def group(self):
    return self.tr("Configurations")

  def groupId(self):
    return "config"

  def createInstance(self):
    return installcomponents()
